@@ -1,5 +1,5 @@
 (* Unison file synchronizer: src/fileinfo.mli *)
-(* Copyright 1999-2007 (see COPYING for details) *)
+(* Copyright 1999-2009, Benjamin C. Pierce (see COPYING for details) *)
 
 type typ = [`ABSENT | `FILE | `DIRECTORY | `SYMLINK]
 val type2string : typ -> string
@@ -8,10 +8,9 @@ type t = { typ : typ; inode : int; ctime : float;
            desc : Props.t; osX : Osx.info}
 
 val get : bool -> Fspath.t -> Path.local -> t
-val set :
-  Fspath.t -> Path.local ->
-  [`Set of Props.t | `Copy of Path.local | `Update of Props.t] ->
-  Props.t -> unit
+val set : Fspath.t -> Path.local ->
+          [`Set of Props.t | `Copy of Path.local | `Update of Props.t] ->
+          Props.t -> unit
 
 (* IF THIS CHANGES, MAKE SURE TO INCREMENT THE ARCHIVE VERSION NUMBER!       *)
 type stamp =
