@@ -1,5 +1,20 @@
 (* Unison file synchronizer: src/pred.ml *)
-(* Copyright 1999-2007 (see COPYING for details) *)
+(* Copyright 1999-2009, Benjamin C. Pierce 
+
+    This program is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*)
+
 
 let debug = Util.debug "pred"
 
@@ -71,9 +86,12 @@ let compile_pattern clause =
     end in
   (compiled, v)
 
-let create name fulldoc =
+let create name ?(advanced=false) fulldoc =
   let pref = 
-    Prefs.create name [] ("add a pattern to the " ^ name ^ " list") fulldoc
+    Prefs.create name []
+      ((if advanced then "!" else "")
+       ^ "add a pattern to the " ^ name ^ " list")
+      fulldoc
       (fun oldList string ->
          ignore (compile_pattern string); (* Check well-formedness *)
         string :: oldList)
