@@ -1,5 +1,20 @@
 (* Unison file synchronizer: src/transport.ml *)
-(* Copyright 1999-2007 (see COPYING for details) *)
+(* Copyright 1999-2009, Benjamin C. Pierce 
+
+    This program is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*)
+
 
 open Common
 open Lwt
@@ -22,7 +37,7 @@ let fileSize uiFrom uiTo =
 
 let maxthreads =
   Prefs.createInt "maxthreads" 20
-    "maximum number of simultaneous file transfers"
+    "!maximum number of simultaneous file transfers"
     ("This preference controls how much concurrency is allowed during"
      ^ " the transport phase.  Normally, it should be set reasonably high "
      ^ "(default is 20) to maximize performance, but when Unison is used "
@@ -72,6 +87,7 @@ let stashCurrentVersions fromRoot toRoot path =
 
 let doAction (fromRoot,toRoot) path fromContents toContents id =
   Lwt_util.resize_region actionReg (Prefs.read maxthreads);
+  Lwt_util.resize_region Files.copyReg (Prefs.read maxthreads);
   Lwt_util.run_in_region actionReg 1 (fun () ->
     if not !Trace.sendLogMsgsToStderr then
       Trace.statusDetail (Path.toString path);
