@@ -1,5 +1,20 @@
 (* Unison file synchronizer: src/uutil.ml *)
-(* Copyright 1999-2007 (see COPYING for details) *)
+(* Copyright 1999-2009, Benjamin C. Pierce 
+
+    This program is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*)
+
 
 (*****************************************************************************)
 (*                      Unison name and version                              *)
@@ -79,6 +94,13 @@ let setProgressPrinter p = progressPrinter := p
 let showProgress i bytes ch =
   if i <> File.dummy then !progressPrinter i bytes ch
 
+let statusPrinter = ref None
+let setUpdateStatusPrinter p = statusPrinter := p
+let showUpdateStatus path =
+  match !statusPrinter with
+    Some f -> f path
+  | None   -> Trace.statusDetail path
+
 (*****************************************************************************)
 (*               Copy bytes from one file_desc to another                    *)
 (*****************************************************************************)
