@@ -1,6 +1,5 @@
-(* $I1: Unison file synchronizer: src/xferhint.mli $ *)
-(* $I2: Last modified by zheyang on Fri, 22 Mar 2002 14:15:11 -0500 $ *)
-(* $I3: Copyright 1999-2002 (see COPYING for details) $ *)
+(* Unison file synchronizer: src/xferhint.mli *)
+(* Copyright 1999-2009, Benjamin C. Pierce (see COPYING for details) *)
 
 (* This module maintains a cache that can be used to map
    an Os.fingerprint to a (Fspath.t * Path.t) naming a file that *may*
@@ -11,10 +10,9 @@
 val xferbycopying: bool Prefs.t
 
 (* Suggest a file that's likely to have a given fingerprint *)
-val lookup: Os.fingerprint -> (Fspath.t * Path.t) option
+val lookup: Os.fullfingerprint -> (Fspath.t * Path.local) option
 
 (* Add, delete, and rename entries *)
-val insertEntry: Fspath.t * Path.t -> Os.fingerprint -> unit
-val deleteEntry: Fspath.t * Path.t -> unit
-val renameEntry: Fspath.t * Path.t -> Fspath.t * Path.t -> unit 
-
+val insertEntry: Fspath.t * Path.local -> Os.fullfingerprint -> unit
+val deleteEntry: Fspath.t * Path.local -> unit
+val renameEntry: Fspath.t * Path.local -> Fspath.t * Path.local -> unit
